@@ -17,8 +17,9 @@ use strict;
 use warnings;
 
 package Clownfish::CFC;
-our $VERSION = '0.004004';
+our $VERSION = '0.004099_001';
 $VERSION = eval $VERSION;
+our $MAJOR_VERSION = 0.004000;
 
 END {
     Clownfish::CFC::Model::Class->_clear_registry();
@@ -26,7 +27,7 @@ END {
 }
 
 use XSLoader;
-BEGIN { XSLoader::load( 'Clownfish::CFC', '0.4.4' ) }
+BEGIN { XSLoader::load( 'Clownfish::CFC', '0.4.99_1' ) }
 
 {
     package Clownfish::CFC::Util;
@@ -106,7 +107,7 @@ BEGIN { XSLoader::load( 'Clownfish::CFC', '0.4.4' ) }
 
 {
     package Clownfish::CFC::Model::Class;
-    BEGIN { push our @ISA, 'Clownfish::CFC::Model::Symbol' }
+    BEGIN { push our @ISA, 'Clownfish::CFC::Base' }
     use Carp;
     use Config;
     use Clownfish::CFC::Util qw(
@@ -123,6 +124,7 @@ BEGIN { XSLoader::load( 'Clownfish::CFC', '0.4.4' ) }
         inert             => undef,
         final             => undef,
         parcel            => undef,
+        abstract          => undef,
         exposure          => 'parcel',
     );
 
@@ -155,8 +157,8 @@ BEGIN { XSLoader::load( 'Clownfish::CFC', '0.4.4' ) }
             = Clownfish::CFC::Model::Parcel->acquire( $args{parcel} );
         return _create(
             @args{
-                qw( parcel exposure class_name nickname micro_sym docucomment
-                    file_spec parent_class_name final inert)
+                qw( parcel exposure class_name nickname docucomment
+                    file_spec parent_class_name final inert abstract)
                 }
         );
     }
@@ -213,15 +215,12 @@ BEGIN { XSLoader::load( 'Clownfish::CFC', '0.4.4' ) }
     use Clownfish::CFC::Util qw( verify_args a_isa_b );
 
     my %new_PARAMS = (
-        return_type    => undef,
-        class_name     => undef,
-        class_nickname => undef,
-        param_list     => undef,
-        micro_sym      => undef,
-        docucomment    => undef,
-        parcel         => undef,
-        inline         => undef,
-        exposure       => undef,
+        return_type => undef,
+        param_list  => undef,
+        name        => undef,
+        docucomment => undef,
+        inline      => undef,
+        exposure    => undef,
     );
 
     sub new {
@@ -229,12 +228,9 @@ BEGIN { XSLoader::load( 'Clownfish::CFC', '0.4.4' ) }
         confess "no subclassing allowed" unless $either eq __PACKAGE__;
         verify_args( \%new_PARAMS, %args ) or confess $@;
         $args{inline} ||= 0;
-        $args{parcel}
-            = Clownfish::CFC::Model::Parcel->acquire( $args{parcel} );
         return _new(
             @args{
-                qw( parcel exposure class_name class_nickname micro_sym
-                    return_type param_list docucomment inline )
+                qw( exposure name return_type param_list docucomment inline )
                 }
         );
     }
@@ -270,7 +266,7 @@ BEGIN { XSLoader::load( 'Clownfish::CFC', '0.4.4' ) }
         for my $class (@{ $self->ordered_classes }) {
             next if !$class->included || $class->inert;
 
-            my $class_name = $class->get_class_name;
+            my $class_name = $class->get_name;
             my $rt_class = Clownfish::Class->fetch_class($class_name)
                 or die("Class $class_name not found");
 
@@ -298,16 +294,14 @@ BEGIN { XSLoader::load( 'Clownfish::CFC', '0.4.4' ) }
     use Carp;
 
     my %new_PARAMS = (
-        return_type    => undef,
-        class_name     => undef,
-        class_nickname => undef,
-        param_list     => undef,
-        macro_sym      => undef,
-        docucomment    => undef,
-        parcel         => undef,
-        abstract       => undef,
-        final          => undef,
-        exposure       => 'parcel',
+        return_type => undef,
+        param_list  => undef,
+        name        => undef,
+        docucomment => undef,
+        class_name  => undef,
+        abstract    => undef,
+        final       => undef,
+        exposure    => 'parcel',
     );
 
     sub new {
@@ -315,13 +309,11 @@ BEGIN { XSLoader::load( 'Clownfish::CFC', '0.4.4' ) }
         verify_args( \%new_PARAMS, %args ) or confess $@;
         confess "no subclassing allowed" unless $either eq __PACKAGE__;
         $args{abstract} ||= 0;
-        $args{parcel}
-            = Clownfish::CFC::Model::Parcel->acquire( $args{parcel} );
-        $args{final} ||= 0;
+        $args{final}    ||= 0;
         return _new(
             @args{
-                qw( parcel exposure class_name class_nickname macro_sym
-                    return_type param_list docucomment final abstract )
+                qw( exposure name return_type param_list docucomment class_name
+                    final abstract )
                 }
         );
     }
@@ -453,21 +445,16 @@ BEGIN { XSLoader::load( 'Clownfish::CFC', '0.4.4' ) }
     use Carp;
 
     my %new_PARAMS = (
-        parcel         => undef,
-        exposure       => undef,
-        class_name     => undef,
-        class_nickname => undef,
-        micro_sym      => undef,
+        exposure => undef,
+        name     => undef,
     );
 
     sub new {
         my ( $either, %args ) = @_;
         verify_args( \%new_PARAMS, %args ) or confess $@;
         confess "no subclassing allowed" unless $either eq __PACKAGE__;
-        $args{parcel}
-            = Clownfish::CFC::Model::Parcel->acquire( $args{parcel} );
         return _new(
-            @args{qw( parcel exposure class_name class_nickname micro_sym )} );
+            @args{qw( exposure name )} );
     }
 }
 
@@ -479,19 +466,19 @@ BEGIN { XSLoader::load( 'Clownfish::CFC', '0.4.4' ) }
     use Carp;
 
     our %new_PARAMS = (
-        const       => undef,
-        specifier   => undef,
-        indirection => undef,
-        parcel      => undef,
-        void        => undef,
-        object      => undef,
-        primitive   => undef,
-        integer     => undef,
-        floating    => undef,
-        string_type => undef,
-        va_list     => undef,
-        arbitrary   => undef,
-        composite   => undef,
+        const        => undef,
+        specifier    => undef,
+        indirection  => undef,
+        parcel       => undef,
+        void         => undef,
+        object       => undef,
+        primitive    => undef,
+        integer      => undef,
+        floating     => undef,
+        cfish_string => undef,
+        va_list      => undef,
+        arbitrary    => undef,
+        composite    => undef,
     );
 
     sub new {
@@ -501,17 +488,17 @@ BEGIN { XSLoader::load( 'Clownfish::CFC', '0.4.4' ) }
         verify_args( \%new_PARAMS, %args ) or confess $@;
 
         my $flags = 0;
-        $flags |= CONST       if $args{const};
-        $flags |= NULLABLE    if $args{nullable};
-        $flags |= VOID        if $args{void};
-        $flags |= OBJECT      if $args{object};
-        $flags |= PRIMITIVE   if $args{primitive};
-        $flags |= INTEGER     if $args{integer};
-        $flags |= FLOATING    if $args{floating};
-        $flags |= STRING_TYPE if $args{string_type};
-        $flags |= VA_LIST     if $args{va_list};
-        $flags |= ARBITRARY   if $args{arbitrary};
-        $flags |= COMPOSITE   if $args{composite};
+        $flags |= CONST        if $args{const};
+        $flags |= NULLABLE     if $args{nullable};
+        $flags |= VOID         if $args{void};
+        $flags |= OBJECT       if $args{object};
+        $flags |= PRIMITIVE    if $args{primitive};
+        $flags |= INTEGER      if $args{integer};
+        $flags |= FLOATING     if $args{floating};
+        $flags |= CFISH_STRING if $args{cfish_string};
+        $flags |= VA_LIST      if $args{va_list};
+        $flags |= ARBITRARY    if $args{arbitrary};
+        $flags |= COMPOSITE    if $args{composite};
 
         my $parcel
             = $args{parcel}
@@ -635,13 +622,10 @@ BEGIN { XSLoader::load( 'Clownfish::CFC', '0.4.4' ) }
     use Carp;
 
     our %new_PARAMS = (
-        type           => undef,
-        micro_sym      => undef,
-        parcel         => undef,
-        exposure       => 'local',
-        class_name     => undef,
-        class_nickname => undef,
-        inert          => undef,
+        type     => undef,
+        name     => undef,
+        exposure => 'local',
+        inert    => undef,
     );
 
     sub new {
@@ -649,12 +633,9 @@ BEGIN { XSLoader::load( 'Clownfish::CFC', '0.4.4' ) }
         confess "no subclassing allowed" unless $either eq __PACKAGE__;
         verify_args( \%new_PARAMS, %args ) or confess $@;
         $args{exposure} ||= 'local';
-        $args{parcel}
-            = Clownfish::CFC::Model::Parcel->acquire( $args{parcel} );
         return _new(
             @args{
-                qw( parcel exposure class_name class_nickname micro_sym type
-                    inert )
+                qw( exposure name type inert )
                 }
         );
     }
@@ -866,16 +847,16 @@ BEGIN { XSLoader::load( 'Clownfish::CFC', '0.4.4' ) }
     }
 
     my %add_constructor_PARAMS = (
-        alias       => undef,
-        initializer => undef,
-        sample      => undef,
-        pod         => undef,
+        alias    => undef,
+        pod_func => undef,
+        sample   => undef,
+        pod      => undef,
     );
 
     sub add_constructor {
         my ( $self, %args ) = @_;
         verify_args( \%add_constructor_PARAMS, %args ) or confess $@;
-        _add_constructor( $self, @args{qw( alias initializer sample pod )} );
+        _add_constructor( $self, @args{qw( alias pod_func sample pod )} );
     }
 }
 
