@@ -20,7 +20,7 @@ package Clownfish::CFC::Perl::Build::Charmonic;
 
 use base qw( Module::Build );
 
-our $VERSION = '0.004004';
+our $VERSION = '0.004099_001';
 $VERSION = eval $VERSION;
 
 use Carp;
@@ -79,6 +79,7 @@ sub ACTION_charmony {
     my @command = (
         $CHARMONIZER_EXE_PATH,
         "--cc=$cc",
+        '--host=perl',
         '--enable-c',
         '--enable-perl',
     );
