@@ -32,7 +32,14 @@
 #include "CFCPerlConstructor.h"
 #include "CFCPerlMethod.h"
 #include "CFCPerlTypeMap.h"
+#include "CFCPerlPod.h"
 #include "CFCBindCore.h"
+#include "CFCDocument.h"
+
+typedef struct CFCPerlPodFile {
+    char *path;
+    char *contents;
+} CFCPerlPodFile;
 
 struct CFCPerl {
     CFCBase base;
@@ -41,6 +48,10 @@ struct CFCPerl {
     char *boot_class;
     char *header;
     char *footer;
+    char *c_header;
+    char *c_footer;
+    char *pod_header;
+    char *pod_footer;
     char *xs_path;
     char *boot_func;
 };
@@ -49,6 +60,12 @@ struct CFCPerl {
 static void
 S_replace_double_colons(char *text, char replacement);
 
+static CFCPerlPodFile*
+S_write_class_pod(CFCPerl *self);
+
+static CFCPerlPodFile*
+S_write_standalone_pod(CFCPerl *self);
+
 static void
 S_write_callbacks_c(CFCPerl *self);
 
@@ -78,6 +95,10 @@ CFCPerl_init(CFCPerl *self, CFCHierarchy *hierarchy, const char *lib_dir,
     self->boot_class = CFCUtil_strdup(boot_class);
     self->header     = CFCUtil_strdup(header);
     self->footer     = CFCUtil_strdup(footer);
+    self->c_header   = CFCUtil_make_c_comment(header);
+    self->c_footer   = CFCUtil_make_c_comment(footer);
+    self->pod_header = CFCUtil_make_perl_comment(header);
+    self->pod_footer = CFCUtil_make_perl_comment(footer);
 
     // Derive path to generated .xs file.
     self->xs_path = CFCUtil_sprintf("%s" CHY_DIR_SEP "%s.xs", lib_dir,
@@ -102,6 +123,10 @@ CFCPerl_destroy(CFCPerl *self) {
     FREEMEM(self->boot_class);
     FREEMEM(self->header);
     FREEMEM(self->footer);
+    FREEMEM(self->c_header);
+    FREEMEM(self->c_footer);
+    FREEMEM(self->pod_header);
+    FREEMEM(self->pod_footer);
     FREEMEM(self->xs_path);
     FREEMEM(self->boot_func);
     CFCBase_destroy((CFCBase*)self);
@@ -124,45 +149,128 @@ S_replace_double_colons(char *text, char replacement) {
 
 char**
 CFCPerl_write_pod(CFCPerl *self) {
+    CFCPerlPodFile *class_pods      = S_write_class_pod(self);
+    CFCPerlPodFile *standalone_pods = S_write_standalone_pod(self);
+
+    size_t max_paths = 0;
+    for (size_t i = 0; class_pods[i].contents; i++)      { max_paths++; }
+    for (size_t i = 0; standalone_pods[i].contents; i++) { max_paths++; }
+    char **pod_paths = (char**)CALLOCATE(max_paths + 1, sizeof(char*));
+
+    // Write out any POD files that have changed.
+    CFCPerlPodFile *file_arrays[2] = {
+        class_pods,
+        standalone_pods
+    };
+    size_t num_written = 0;
+    for (size_t j = 0; j < 2; ++j) {
+        CFCPerlPodFile *pod_files = file_arrays[j];
+
+        for (size_t i = 0; pod_files[i].contents; i++) {
+            char *pod      = pod_files[i].contents;
+            char *pod_path = pod_files[i].path;
+            char *pod_dir  = CFCUtil_strdup(pod_path);
+
+            char *last_dir_sep = strrchr(pod_dir, CHY_DIR_SEP_CHAR);
+            if (last_dir_sep) {
+                *last_dir_sep = '\0';
+                if (!CFCUtil_make_path(pod_dir)) {
+                    CFCUtil_die("Can't make path %s", pod_dir);
+                }
+            }
+
+            if (CFCUtil_write_if_changed(pod_path, pod, strlen(pod))) {
+                pod_paths[num_written] = pod_path;
+                num_written++;
+            }
+            else {
+                FREEMEM(pod_path);
+            }
+
+            FREEMEM(pod);
+            FREEMEM(pod_dir);
+        }
+
+        FREEMEM(pod_files);
+    }
+    pod_paths[num_written] = NULL;
+
+    return pod_paths;
+}
+
+static CFCPerlPodFile*
+S_write_class_pod(CFCPerl *self) {
     CFCPerlClass **registry  = CFCPerlClass_registry();
     size_t num_registered = 0;
     while (registry[num_registered] != NULL) { num_registered++; }
-    char     **pod_paths = (char**)CALLOCATE(num_registered + 1, sizeof(char*));
-    char     **pods      = (char**)CALLOCATE(num_registered + 1, sizeof(char*));
-    size_t     count     = 0;
+    CFCPerlPodFile *pod_files
+        = (CFCPerlPodFile*)CALLOCATE(num_registered + 1,
+                                     sizeof(CFCPerlPodFile));
+    size_t count = 0;
 
     // Generate POD, but don't write.  That way, if there's an error while
     // generating pod, we leak memory but don't clutter up the file system.
     for (size_t i = 0; i < num_registered; i++) {
         const char *class_name = CFCPerlClass_get_class_name(registry[i]);
-        char *pod = CFCPerlClass_create_pod(registry[i]);
-        if (!pod) { continue; }
+        char *raw_pod = CFCPerlClass_create_pod(registry[i]);
+        if (!raw_pod) { continue; }
+        char *pod = CFCUtil_sprintf("%s\n%s%s", self->pod_header, raw_pod,
+                                    self->pod_footer);
         char *pod_path = CFCUtil_sprintf("%s" CHY_DIR_SEP "%s.pod",
                                          self->lib_dir, class_name);
         S_replace_double_colons(pod_path, CHY_DIR_SEP_CHAR);
 
-        pods[count] = pod;
-        pod_paths[count] = pod_path;
+        pod_files[count].contents = pod;
+        pod_files[count].path     = pod_path;
         count++;
+
+        FREEMEM(raw_pod);
+    }
+    pod_files[count].contents = NULL;
+    pod_files[count].path     = NULL;
+
+    return pod_files;
+}
+
+static CFCPerlPodFile*
+S_write_standalone_pod(CFCPerl *self) {
+    CFCDocument **docs = CFCDocument_get_registry();
+    size_t num_pod_files = 0;
+    while (docs[num_pod_files]) { num_pod_files++; }
+    size_t alloc_size = (num_pod_files + 1) * sizeof(CFCPerlPodFile);
+    CFCPerlPodFile *pod_files = (CFCPerlPodFile*)MALLOCATE(alloc_size);
+
+    for (size_t i = 0; i < num_pod_files; i++) {
+        CFCDocument *doc = docs[i];
+        char *md      = CFCDocument_get_contents(doc);
+        char *raw_pod = CFCPerlPod_md_to_pod(md, NULL, 1);
+
+        const char *pattern =
+            "%s"
+            "\n"
+            "=encoding utf8\n"
+            "\n"
+            "%s"
+            "\n"
+            "%s";
+        char *pod = CFCUtil_sprintf(pattern, self->pod_header, raw_pod,
+                                    self->pod_footer);
+
+        const char *path_part = CFCDocument_get_path_part(doc);
+        char *pod_path = CFCUtil_sprintf("%s" CHY_DIR_SEP "%s.pod",
+                                         self->lib_dir, path_part);
+
+        pod_files[i].contents = pod;
+        pod_files[i].path     = pod_path;
+
+        FREEMEM(raw_pod);
+        FREEMEM(md);
     }
 
-    // Write out any POD files that have changed.
-    size_t num_written = 0;
-    for (size_t i = 0; i < count; i++) {
-        char *pod      = pods[i];
-        char *pod_path = pod_paths[i];
-        if (CFCUtil_write_if_changed(pod_path, pod, strlen(pod))) {
-            pod_paths[num_written] = pod_path;
-            num_written++;
-        }
-        else {
-            FREEMEM(pod_path);
-        }
-        FREEMEM(pod);
-    }
-    pod_paths[num_written] = NULL;
+    pod_files[num_pod_files].contents = NULL;
+    pod_files[num_pod_files].path     = NULL;
 
-    return pod_paths;
+    return pod_files;
 }
 
 static void
@@ -196,8 +304,8 @@ S_write_boot_h(CFCPerl *self) {
         "\n"
         "%s\n";
     char *content
-        = CFCUtil_sprintf(pattern, self->header, guard, guard, self->boot_func,
-                          guard, self->footer);
+        = CFCUtil_sprintf(pattern, self->c_header, guard, guard,
+                          self->boot_func, guard, self->c_footer);
 
     const char *inc_dest = CFCHierarchy_get_include_dest(self->hierarchy);
     char *boot_h_path = CFCUtil_sprintf("%s" CHY_DIR_SEP "boot.h", inc_dest);
@@ -218,8 +326,12 @@ S_write_boot_c(CFCPerl *self) {
     char *isa_pushes      = CFCUtil_strdup("");
 
     for (size_t i = 0; parcels[i]; ++i) {
-        if (!CFCParcel_included(parcels[i])) {
-            const char *prefix = CFCParcel_get_prefix(parcels[i]);
+        CFCParcel *parcel = parcels[i];
+
+        if (!CFCParcel_included(parcel)) {
+            const char *prefix = CFCParcel_get_prefix(parcel);
+            pound_includes = CFCUtil_cat(pound_includes, "#include \"", prefix,
+                                         "parcel.h\"\n", NULL);
             bootstrap_code = CFCUtil_cat(bootstrap_code, "    ", prefix,
                                          "bootstrap_parcel();\n", NULL);
         }
@@ -227,14 +339,9 @@ S_write_boot_c(CFCPerl *self) {
 
     for (size_t i = 0; ordered[i] != NULL; i++) {
         CFCClass *klass = ordered[i];
-        if (CFCClass_included(klass)) { continue; }
+        if (CFCClass_included(klass) || CFCClass_inert(klass)) { continue; }
 
-        const char *class_name = CFCClass_get_class_name(klass);
-        const char *include_h  = CFCClass_include_h(klass);
-        pound_includes = CFCUtil_cat(pound_includes, "#include \"",
-                                     include_h, "\"\n", NULL);
-
-        if (CFCClass_inert(klass)) { continue; }
+        const char *class_name = CFCClass_get_name(klass);
 
         // Add aliases for selected KinoSearch classes which allow old indexes
         // to be read.
@@ -264,7 +371,7 @@ S_write_boot_c(CFCPerl *self) {
 
         CFCClass *parent = CFCClass_get_parent(klass);
         if (parent) {
-            const char *parent_class_name = CFCClass_get_class_name(parent);
+            const char *parent_class_name = CFCClass_get_name(parent);
             isa_pushes
                 = CFCUtil_cat(isa_pushes, "    isa = get_av(\"",
                               class_name, "::ISA\", 1);\n", NULL);
@@ -277,21 +384,22 @@ S_write_boot_c(CFCPerl *self) {
     const char pattern[] =
         "%s\n"
         "\n"
-        "#include \"cfish_parcel.h\"\n"
-        "\n"
-        "/* Avoid conflicts with Clownfish bool type. */\n"
-        "#define HAS_BOOL\n"
-        "#include \"EXTERN.h\"\n"
-        "#include \"perl.h\"\n"
-        "#include \"XSUB.h\"\n"
-        "\n"
         "#include \"boot.h\"\n"
         "#include \"Clownfish/String.h\"\n"
         "#include \"Clownfish/Class.h\"\n"
         "%s\n"
         "\n"
+        "/* Avoid conflicts with Clownfish bool type. */\n"
+        "#define HAS_BOOL\n"
+        "#define PERL_NO_GET_CONTEXT\n"
+        "#include \"EXTERN.h\"\n"
+        "#include \"perl.h\"\n"
+        "#include \"XSUB.h\"\n"
+        "\n"
         "void\n"
         "%s() {\n"
+        "    dTHX;\n"
+        "\n"
         "%s"
         "\n"
         "%s"
@@ -303,9 +411,9 @@ S_write_boot_c(CFCPerl *self) {
         "%s\n"
         "\n";
     char *content
-        = CFCUtil_sprintf(pattern, self->header, pound_includes,
+        = CFCUtil_sprintf(pattern, self->c_header, pound_includes,
                           self->boot_func, bootstrap_code, alias_adds,
-                          isa_pushes, self->footer);
+                          isa_pushes, self->c_footer);
 
     const char *src_dest = CFCHierarchy_get_source_dest(self->hierarchy);
     char *boot_c_path = CFCUtil_sprintf("%s" CHY_DIR_SEP "boot.c", src_dest);
@@ -341,7 +449,7 @@ CFCPerl_write_hostdefs(CFCPerl *self) {
         "\n"
         "%s\n";
     char *content
-        = CFCUtil_sprintf(pattern, self->header, self->footer);
+        = CFCUtil_sprintf(pattern, self->c_header, self->c_footer);
 
     // Unlink then write file.
     const char *inc_dest = CFCHierarchy_get_include_dest(self->hierarchy);
@@ -381,9 +489,9 @@ S_xs_file_contents(CFCPerl *self, const char *generated_xs,
         "\n"
         "%s";
     char *contents
-        = CFCUtil_sprintf(pattern, self->header, generated_xs,
+        = CFCUtil_sprintf(pattern, self->c_header, generated_xs,
                           self->boot_class, self->boot_class, self->boot_func,
-                          xs_init, hand_rolled_xs, self->footer);
+                          xs_init, hand_rolled_xs, self->c_footer);
 
     return contents;
 }
@@ -424,6 +532,7 @@ CFCPerl_write_bindings(CFCPerl *self) {
     // Pound-includes for generated headers.
     for (size_t i = 0; ordered[i] != NULL; i++) {
         CFCClass *klass = ordered[i];
+        if (CFCClass_included(klass)) { continue; }
         const char *include_h = CFCClass_include_h(klass);
         generated_xs = CFCUtil_cat(generated_xs, "#include \"", include_h,
                                    "\"\n", NULL);
@@ -441,13 +550,16 @@ CFCPerl_write_bindings(CFCPerl *self) {
             CFCPerlSub *xsub = (CFCPerlSub*)constructors[j];
 
             // Add the XSUB function definition.
-            char *xsub_def = CFCPerlConstructor_xsub_def(constructors[j]);
+            char *xsub_def
+                = CFCPerlConstructor_xsub_def(constructors[j], klass);
             generated_xs = CFCUtil_cat(generated_xs, xsub_def, "\n",
                                        NULL);
             FREEMEM(xsub_def);
 
             // Add XSUB initialization at boot.
             xs_init = S_add_xs_init(xs_init, xsub);
+
+            CFCBase_decref((CFCBase*)constructors[j]);
         }
         FREEMEM(constructors);
 
@@ -457,13 +569,15 @@ CFCPerl_write_bindings(CFCPerl *self) {
             CFCPerlSub *xsub = (CFCPerlSub*)methods[j];
 
             // Add the XSUB function definition.
-            char *xsub_def = CFCPerlMethod_xsub_def(methods[j]);
+            char *xsub_def = CFCPerlMethod_xsub_def(methods[j], klass);
             generated_xs = CFCUtil_cat(generated_xs, xsub_def, "\n",
                                        NULL);
             FREEMEM(xsub_def);
 
             // Add XSUB initialization at boot.
             xs_init = S_add_xs_init(xs_init, xsub);
+
+            CFCBase_decref((CFCBase*)methods[j]);
         }
         FREEMEM(methods);
     }
@@ -489,25 +603,46 @@ CFCPerl_write_bindings(CFCPerl *self) {
 
 void
 CFCPerl_write_callbacks(CFCPerl *self) {
-    CFCBindCore *core_binding
-        = CFCBindCore_new(self->hierarchy, self->header, self->footer);
-    CFCBindCore_write_callbacks_h(core_binding);
-    CFCBase_decref((CFCBase*)core_binding);
-
     S_write_callbacks_c(self);
 }
 
 static void
 S_write_callbacks_c(CFCPerl *self) {
-    CFCClass **ordered = CFCHierarchy_ordered_classes(self->hierarchy);
+    CFCClass **ordered  = CFCHierarchy_ordered_classes(self->hierarchy);
+    char      *includes = CFCUtil_strdup("");
+    char      *cb_defs  = CFCUtil_strdup("");
+
+    for (size_t i = 0; ordered[i] != NULL; i++) {
+        CFCClass *klass = ordered[i];
+        if (CFCClass_included(klass) || CFCClass_inert(klass)) { continue; }
+
+        const char *include_h = CFCClass_include_h(klass);
+        includes = CFCUtil_cat(includes, "#include \"", include_h,
+                               "\"\n", NULL);
+
+        CFCMethod **fresh_methods = CFCClass_fresh_methods(klass);
+        for (int meth_num = 0; fresh_methods[meth_num] != NULL; meth_num++) {
+            CFCMethod *method = fresh_methods[meth_num];
+
+            // Define callback.
+            if (CFCMethod_novel(method) && !CFCMethod_final(method)) {
+                char *cb_def = CFCPerlMethod_callback_def(method, klass);
+                cb_defs = CFCUtil_cat(cb_defs, cb_def, "\n", NULL);
+                FREEMEM(cb_def);
+            }
+        }
+    }
+
     static const char pattern[] =
         "%s"
         "\n"
         "#include \"XSBind.h\"\n"
-        "#include \"callbacks.h\"\n"
+        "#include \"Clownfish/Err.h\"\n"
+        "#include \"Clownfish/Obj.h\"\n"
+        "%s"
         "\n"
         "static void\n"
-        "S_finish_callback_void(const char *meth_name) {\n"
+        "S_finish_callback_void(pTHX_ const char *meth_name) {\n"
         "    int count = call_method(meth_name, G_VOID | G_DISCARD);\n"
         "    if (count != 0) {\n"
         "        CFISH_THROW(CFISH_ERR, \"Bad callback to '%%s': %%i32\",\n"
@@ -518,7 +653,7 @@ S_write_callbacks_c(CFCPerl *self) {
         "}\n"
         "\n"
         "static CFISH_INLINE SV*\n"
-        "SI_do_callback_sv(const char *meth_name) {\n"
+        "SI_do_callback_sv(pTHX_ const char *meth_name) {\n"
         "    int count = call_method(meth_name, G_SCALAR);\n"
         "    if (count != 1) {\n"
         "        CFISH_THROW(CFISH_ERR, \"Bad callback to '%%s': %%i32\",\n"
@@ -531,8 +666,8 @@ S_write_callbacks_c(CFCPerl *self) {
         "}\n"
         "\n"
         "static int64_t\n"
-        "S_finish_callback_i64(const char *meth_name) {\n"
-        "    SV *return_sv = SI_do_callback_sv(meth_name);\n"
+        "S_finish_callback_i64(pTHX_ const char *meth_name) {\n"
+        "    SV *return_sv = SI_do_callback_sv(aTHX_ meth_name);\n"
         "    int64_t retval;\n"
         "    if (sizeof(IV) == 8) {\n"
         "        retval = (int64_t)SvIV(return_sv);\n"
@@ -554,8 +689,8 @@ S_write_callbacks_c(CFCPerl *self) {
         "}\n"
         "\n"
         "static double\n"
-        "S_finish_callback_f64(const char *meth_name) {\n"
-        "    SV *return_sv = SI_do_callback_sv(meth_name);\n"
+        "S_finish_callback_f64(pTHX_ const char *meth_name) {\n"
+        "    SV *return_sv = SI_do_callback_sv(aTHX_ meth_name);\n"
         "    double retval = SvNV(return_sv);\n"
         "    FREETMPS;\n"
         "    LEAVE;\n"
@@ -563,41 +698,26 @@ S_write_callbacks_c(CFCPerl *self) {
         "}\n"
         "\n"
         "static cfish_Obj*\n"
-        "S_finish_callback_obj(void *vself, const char *meth_name,\n"
+        "S_finish_callback_obj(pTHX_ void *vself, const char *meth_name,\n"
         "                      int nullable) {\n"
-        "    SV *return_sv = SI_do_callback_sv(meth_name);\n"
-        "    cfish_Obj *retval = XSBind_perl_to_cfish(return_sv);\n"
+        "    SV *return_sv = SI_do_callback_sv(aTHX_ meth_name);\n"
+        "    cfish_Obj *retval\n"
+        "        = XSBind_perl_to_cfish_nullable(aTHX_ return_sv, CFISH_OBJ);\n"
         "    FREETMPS;\n"
         "    LEAVE;\n"
         "    if (!nullable && !retval) {\n"
         "        CFISH_THROW(CFISH_ERR, \"%%o#%%s cannot return NULL\",\n"
-        "                    CFISH_Obj_Get_Class_Name((cfish_Obj*)vself),\n"
+        "                    cfish_Obj_get_class_name((cfish_Obj*)vself),\n"
         "                    meth_name);\n"
         "    }\n"
         "    return retval;\n"
         "}\n"
+	"\n"
+	"%s" // Callback definitions.
+	"%s" // Footer.
         "\n";
-    char *content = CFCUtil_sprintf(pattern, self->header);
-
-    for (size_t i = 0; ordered[i] != NULL; i++) {
-        CFCClass *klass = ordered[i];
-        if (CFCClass_included(klass) || CFCClass_inert(klass)) { continue; }
-
-        CFCMethod **fresh_methods = CFCClass_fresh_methods(klass);
-        for (int meth_num = 0; fresh_methods[meth_num] != NULL; meth_num++) {
-            CFCMethod *method = fresh_methods[meth_num];
-
-            // Define callback.
-            if (CFCMethod_novel(method) && !CFCMethod_final(method)) {
-                char *cb_def = CFCPerlMethod_callback_def(method);
-                content = CFCUtil_cat(content, cb_def, "\n", NULL);
-                FREEMEM(cb_def);
-            }
-        }
-        FREEMEM(fresh_methods);
-    }
-
-    content = CFCUtil_cat(content, self->footer, NULL);
+    char *content = CFCUtil_sprintf(pattern, self->c_header, includes, cb_defs,
+                                    self->c_footer);
 
     // Write if changed.
     const char *src_dest = CFCHierarchy_get_source_dest(self->hierarchy);
@@ -607,6 +727,8 @@ S_write_callbacks_c(CFCPerl *self) {
 
     FREEMEM(filepath);
     FREEMEM(content);
+    FREEMEM(cb_defs);
+    FREEMEM(includes);
     FREEMEM(ordered);
 }
 
