@@ -23,6 +23,7 @@
 #include "CFCPerlMethod.h"
 #include "CFCUtil.h"
 #include "CFCClass.h"
+#include "CFCFunction.h"
 #include "CFCMethod.h"
 #include "CFCSymbol.h"
 #include "CFCType.h"
@@ -38,33 +39,19 @@ struct CFCPerlMethod {
 
 // Return the main chunk of the code for the xsub.
 static char*
-S_xsub_body(CFCPerlMethod *self);
+S_xsub_body(CFCPerlMethod *self, CFCClass *klass);
 
 // Create an assignment statement for extracting $self from the Perl stack.
 static char*
-S_self_assign_statement(CFCPerlMethod *self, CFCType *type);
+S_self_assign_statement(CFCPerlMethod *self);
 
 // Return code for an xsub which uses labeled params.
 static char*
-S_xsub_def_labeled_params(CFCPerlMethod *self);
+S_xsub_def_labeled_params(CFCPerlMethod *self, CFCClass *klass);
 
 // Return code for an xsub which uses positional args.
 static char*
-S_xsub_def_positional_args(CFCPerlMethod *self);
-
-/* Take a NULL-terminated list of CFCVariables and build up a string of
- * directives like:
- *
- *     UNUSED_VAR(var1);
- *     UNUSED_VAR(var2);
- */
-static char*
-S_build_unused_vars(CFCVariable **vars);
-
-/* Create an unreachable return statement if necessary, in order to thwart
- * compiler warnings. */
-static char*
-S_maybe_unreachable(CFCType *return_type);
+S_xsub_def_positional_args(CFCPerlMethod *self, CFCClass *klass);
 
 /* Generate code which converts C types to Perl types and pushes arguments
  * onto the Perl stack.
@@ -82,23 +69,23 @@ S_callback_refcount_mods(CFCMethod *method);
  * compile-time.
  */
 static char*
-S_invalid_callback_def(CFCMethod *method);
+S_invalid_callback_body(CFCMethod *method);
 
 // Create a callback for a method which operates in a void context.
 static char*
-S_void_callback_def(CFCMethod *method, const char *callback_start,
-                    const char *refcount_mods);
+S_void_callback_body(CFCMethod *method, const char *callback_start,
+                     const char *refcount_mods);
 
 // Create a callback which returns a primitive type.
 static char*
-S_primitive_callback_def(CFCMethod *method, const char *callback_start,
-                         const char *refcount_mods);
+S_primitive_callback_body(CFCMethod *method, const char *callback_start,
+                          const char *refcount_mods);
 
 /* Create a callback which returns an object type -- either a generic object or
  * a string. */
 static char*
-S_obj_callback_def(CFCMethod *method, const char *callback_start,
-                   const char *refcount_mods);
+S_obj_callback_body(CFCMethod *method, const char *callback_start,
+                    const char *refcount_mods);
 
 static const CFCMeta CFCPERLMETHOD_META = {
     "Clownfish::CFC::Binding::Perl::Method",
@@ -107,16 +94,16 @@ static const CFCMeta CFCPERLMETHOD_META = {
 };
 
 CFCPerlMethod*
-CFCPerlMethod_new(CFCMethod *method) {
+CFCPerlMethod_new(CFCClass *klass, CFCMethod *method) {
     CFCPerlMethod *self
         = (CFCPerlMethod*)CFCBase_allocate(&CFCPERLMETHOD_META);
-    return CFCPerlMethod_init(self, method);
+    return CFCPerlMethod_init(self, klass, method);
 }
 
 CFCPerlMethod*
-CFCPerlMethod_init(CFCPerlMethod *self, CFCMethod *method) {
+CFCPerlMethod_init(CFCPerlMethod *self, CFCClass *klass, CFCMethod *method) {
     CFCParamList *param_list = CFCMethod_get_param_list(method);
-    const char *class_name = CFCMethod_get_class_name(method);
+    const char *class_name = CFCClass_get_name(klass);
     int use_labeled_params = CFCParamList_num_vars(param_list) > 2
                              ? 1 : 0;
 
@@ -143,7 +130,7 @@ CFCPerlMethod_perl_name(CFCMethod *method) {
     }
 
     // Derive Perl name by lowercasing.
-    const char *name      = CFCMethod_get_macro_sym(method);
+    const char *name      = CFCMethod_get_name(method);
     char       *perl_name = CFCUtil_strdup(name);
     for (size_t i = 0; perl_name[i] != '\0'; i++) {
         perl_name[i] = tolower(perl_name[i]);
@@ -153,30 +140,23 @@ CFCPerlMethod_perl_name(CFCMethod *method) {
 }
 
 char*
-CFCPerlMethod_xsub_def(CFCPerlMethod *self) {
+CFCPerlMethod_xsub_def(CFCPerlMethod *self, CFCClass *klass) {
     if (self->sub.use_labeled_params) {
-        return S_xsub_def_labeled_params(self);
+        return S_xsub_def_labeled_params(self, klass);
     }
     else {
-        return S_xsub_def_positional_args(self);
+        return S_xsub_def_positional_args(self, klass);
     }
 }
 
 static char*
-S_xsub_body(CFCPerlMethod *self) {
+S_xsub_body(CFCPerlMethod *self, CFCClass *klass) {
     CFCMethod    *method     = self->method;
     CFCParamList *param_list = CFCMethod_get_param_list(method);
     CFCVariable **arg_vars   = CFCParamList_get_variables(param_list);
     char         *name_list  = CFCPerlSub_arg_name_list((CFCPerlSub*)self);
     char         *body       = CFCUtil_strdup("");
 
-    CFCParcel *parcel = CFCMethod_get_parcel(method);
-    const char *class_name = CFCMethod_get_class_name(method);
-    CFCClass *klass = CFCClass_fetch_singleton(parcel, class_name);
-    if (!klass) {
-        CFCUtil_die("Can't find a CFCClass for '%s'", class_name);
-    }
-
     // Extract the method function pointer.
     char *full_meth = CFCMethod_full_method_sym(method, klass);
     char *method_ptr
@@ -187,12 +167,19 @@ S_xsub_body(CFCPerlMethod *self) {
     FREEMEM(method_ptr);
 
     // Compensate for functions which eat refcounts.
+    // It would be more efficient to convert decremented arguments
+    // by calling XSBind_perl_to_cfish without noinc.
     for (int i = 0; arg_vars[i] != NULL; i++) {
         CFCVariable *var = arg_vars[i];
         CFCType     *type = CFCVariable_get_type(var);
         if (CFCType_is_object(type) && CFCType_decremented(type)) {
-            body = CFCUtil_cat(body, "CFISH_INCREF(arg_",
-                               CFCVariable_micro_sym(var), ");\n    ", NULL);
+            const char *name   = CFCVariable_get_name(var);
+            const char *type_c = CFCType_to_c(type);
+            const char *pattern =
+                "arg_%s = (%s)CFISH_INCREF(arg_%s);\n    ";
+            char *statement = CFCUtil_sprintf(pattern, name, type_c, name);
+            body = CFCUtil_cat(body, statement, NULL);
+            FREEMEM(statement);
         }
     }
 
@@ -228,35 +215,39 @@ S_xsub_body(CFCPerlMethod *self) {
 
 // Create an assignment statement for extracting $self from the Perl stack.
 static char*
-S_self_assign_statement(CFCPerlMethod *self, CFCType *type) {
-    (void)self; // unused
+S_self_assign_statement(CFCPerlMethod *self) {
+    CFCParamList *param_list = CFCMethod_get_param_list(self->method);
+    CFCVariable **vars = CFCParamList_get_variables(param_list);
+    CFCType *type = CFCVariable_get_type(vars[0]);
+    const char *self_name = CFCVariable_get_name(vars[0]);
     const char *type_c = CFCType_to_c(type);
     if (!CFCType_is_object(type)) {
         CFCUtil_die("Not an object type: %s", type_c);
     }
     const char *class_var = CFCType_get_class_var(type);
-    char pattern[] = "arg_self = (%s)XSBind_sv_to_cfish_obj(ST(0), %s, NULL);";
-    char *statement = CFCUtil_sprintf(pattern, type_c, class_var);
+    char pattern[] = "arg_%s = (%s)XSBind_perl_to_cfish_noinc("
+                     "aTHX_ ST(0), %s, NULL);";
+    char *statement = CFCUtil_sprintf(pattern, self_name, type_c, class_var);
 
     return statement;
 }
 
 static char*
-S_xsub_def_labeled_params(CFCPerlMethod *self) {
+S_xsub_def_labeled_params(CFCPerlMethod *self, CFCClass *klass) {
     CFCMethod *method = self->method;
     const char *c_name = self->sub.c_name;
     CFCParamList *param_list = self->sub.param_list;
     CFCVariable **arg_vars   = CFCParamList_get_variables(param_list);
     CFCVariable *self_var    = arg_vars[0];
-    CFCType     *self_type   = CFCVariable_get_type(self_var);
     CFCType     *return_type = CFCMethod_get_return_type(method);
-    const char  *self_type_c    = CFCType_to_c(self_type);
-    const char  *self_micro_sym = CFCVariable_micro_sym(self_var);
-    char *arg_decls    = CFCPerlSub_arg_declarations((CFCPerlSub*)self);
-    char *meth_type_c  = CFCMethod_full_typedef(method, NULL);
-    char *self_assign  = S_self_assign_statement(self, self_type);
-    char *allot_params = CFCPerlSub_build_allot_params((CFCPerlSub*)self);
-    char *body         = S_xsub_body(self);
+    size_t num_vars = CFCParamList_num_vars(param_list);
+    const char  *self_name   = CFCVariable_get_name(self_var);
+    char *param_specs = CFCPerlSub_build_param_specs((CFCPerlSub*)self, 1);
+    char *arg_decls   = CFCPerlSub_arg_declarations((CFCPerlSub*)self, 0);
+    char *meth_type_c = CFCMethod_full_typedef(method, klass);
+    char *self_assign = S_self_assign_statement(self);
+    char *arg_assigns = CFCPerlSub_arg_assignments((CFCPerlSub*)self);
+    char *body        = S_xsub_body(self, klass);
 
     char *retval_decl;
     if (CFCType_is_void(return_type)) {
@@ -271,65 +262,78 @@ S_xsub_def_labeled_params(CFCPerlMethod *self) {
         "XS(%s);\n"
         "XS(%s) {\n"
         "    dXSARGS;\n"
-        "    %s arg_self;\n"
-        "%s"
+        "%s"        // param_specs
+        "    int32_t locations[%d];\n"
+        "    SV *sv;\n"
+        "%s"        // arg_decls
         "    %s method;\n"
-        "    bool args_ok;\n"
         "%s"
         "\n"
         "    CFISH_UNUSED_VAR(cv);\n"
-        "    if (items < 1) { CFISH_THROW(CFISH_ERR, \"Usage: %%s(%s, ...)\",  GvNAME(CvGV(cv))); }\n"
+        "    if (items < 1) {\n"
+        "        XSBind_invalid_args_error(aTHX_ cv, \"%s, ...\");\n"
+        "    }\n"
         "    SP -= items;\n"
         "\n"
-        "    /* Extract vars from Perl stack. */\n"
-        "    %s\n"
-        "    %s\n"
+        "    /* Locate args on Perl stack. */\n"
+        "    XSBind_locate_args(aTHX_ &ST(0), 1, items, param_specs,\n"
+        "                       locations, %d);\n"
+        "    %s\n"  // self_assign
+        "%s"        // arg_assigns
         "\n"
         "    /* Execute */\n"
-        "    %s\n"
+        "    %s\n"  // body
         "}\n";
     char *xsub_def
-        = CFCUtil_sprintf(pattern, c_name, c_name, self_type_c, arg_decls,
-                          meth_type_c, retval_decl, self_micro_sym,
-                          allot_params, self_assign, body);
+        = CFCUtil_sprintf(pattern, c_name, c_name, param_specs, num_vars - 1,
+                          arg_decls, meth_type_c, retval_decl, self_name,
+                          num_vars - 1, self_assign, arg_assigns, body);
 
+    FREEMEM(param_specs);
     FREEMEM(arg_decls);
     FREEMEM(meth_type_c);
     FREEMEM(self_assign);
-    FREEMEM(allot_params);
+    FREEMEM(arg_assigns);
     FREEMEM(body);
     FREEMEM(retval_decl);
     return xsub_def;
 }
 
 static char*
-S_xsub_def_positional_args(CFCPerlMethod *self) {
+S_xsub_def_positional_args(CFCPerlMethod *self, CFCClass *klass) {
     CFCMethod *method = self->method;
     CFCParamList *param_list = CFCMethod_get_param_list(method);
     CFCVariable **arg_vars = CFCParamList_get_variables(param_list);
-    CFCVariable *self_var    = arg_vars[0];
-    CFCType     *self_type   = CFCVariable_get_type(self_var);
     CFCType     *return_type = CFCMethod_get_return_type(method);
-    const char  *self_type_c = CFCType_to_c(self_type);
     const char **arg_inits = CFCParamList_get_initial_values(param_list);
-    unsigned num_vars = (unsigned)CFCParamList_num_vars(param_list);
-    char *arg_decls   = CFCPerlSub_arg_declarations((CFCPerlSub*)self);
-    char *meth_type_c = CFCMethod_full_typedef(method, NULL);
-    char *self_assign = S_self_assign_statement(self, self_type);
-    char *body        = S_xsub_body(self);
+    size_t num_vars = CFCParamList_num_vars(param_list);
+    char *arg_decls   = CFCPerlSub_arg_declarations((CFCPerlSub*)self, 0);
+    char *meth_type_c = CFCMethod_full_typedef(method, klass);
+    char *self_assign = S_self_assign_statement(self);
+    char *arg_assigns = CFCPerlSub_arg_assignments((CFCPerlSub*)self);
+    char *body        = S_xsub_body(self, klass);
 
     // Determine how many args are truly required and build an error check.
-    unsigned min_required = 0;
-    for (unsigned i = 0; i < num_vars; i++) {
+    size_t min_required = 0;
+    for (size_t i = 0; i < num_vars; i++) {
         if (arg_inits[i] == NULL) {
             min_required = i + 1;
         }
     }
+    char *num_args_cond;
+    if (min_required < num_vars) {
+        const char cond_pattern[] = "items < %u || items > %u";
+        num_args_cond = CFCUtil_sprintf(cond_pattern, (unsigned)min_required,
+                                        (unsigned)num_vars);
+    }
+    else {
+        num_args_cond = CFCUtil_sprintf("items != %u", (unsigned)num_vars);
+    }
     char *xs_name_list = num_vars > 0
-                         ? CFCUtil_strdup(CFCVariable_micro_sym(arg_vars[0]))
+                         ? CFCUtil_strdup(CFCVariable_get_name(arg_vars[0]))
                          : CFCUtil_strdup("");
-    for (unsigned i = 1; i < num_vars; i++) {
-        const char *var_name = CFCVariable_micro_sym(arg_vars[i]);
+    for (size_t i = 1; i < num_vars; i++) {
+        const char *var_name = CFCVariable_get_name(arg_vars[i]);
         if (i < min_required) {
             xs_name_list = CFCUtil_cat(xs_name_list, ", ", var_name, NULL);
         }
@@ -338,51 +342,6 @@ S_xsub_def_positional_args(CFCPerlMethod *self) {
                                        NULL);
         }
     }
-    const char num_args_pattern[] =
-        "if (items %s %u) { CFISH_THROW(CFISH_ERR, \"Usage: %%s(%s)\", GvNAME(CvGV(cv))); }";
-    char *num_args_check;
-    if (min_required < num_vars) {
-        num_args_check = CFCUtil_sprintf(num_args_pattern, "<", min_required,
-                                         xs_name_list);
-    }
-    else {
-        num_args_check = CFCUtil_sprintf(num_args_pattern, "!=", num_vars,
-                                         xs_name_list);
-    }
-
-    // Var assignments.
-    char *var_assignments = CFCUtil_strdup("");
-    for (unsigned i = 1; i < num_vars; i++) {
-        CFCVariable *var = arg_vars[i];
-        const char  *val = arg_inits[i];
-        const char  *var_name = CFCVariable_micro_sym(var);
-        CFCType     *var_type = CFCVariable_get_type(var);
-        const char  *type_c   = CFCType_to_c(var_type);
-
-        char perl_stack_var[30];
-        sprintf(perl_stack_var, "ST(%u)", i);
-        char *conversion
-            = CFCPerlTypeMap_from_perl(var_type, perl_stack_var);
-        if (!conversion) {
-            CFCUtil_die("Can't map type '%s'", type_c);
-        }
-        if (val) {
-            char pattern[] =
-                "\n    arg_%s = ( items >= %u && XSBind_sv_defined(ST(%u)) )"
-                " ? %s : %s;";
-            char *statement = CFCUtil_sprintf(pattern, var_name, i, i,
-                                              conversion, val);
-            var_assignments
-                = CFCUtil_cat(var_assignments, statement, NULL);
-            FREEMEM(statement);
-        }
-        else {
-            var_assignments
-                = CFCUtil_cat(var_assignments, "\n    arg_", var_name, " = ",
-                              conversion, ";", NULL);
-        }
-        FREEMEM(conversion);
-    }
 
     char *retval_decl;
     if (CFCType_is_void(return_type)) {
@@ -397,105 +356,101 @@ S_xsub_def_positional_args(CFCPerlMethod *self) {
         "XS(%s);\n"
         "XS(%s) {\n"
         "    dXSARGS;\n"
-        "    %s arg_self;\n"
-        "%s"
+        "    SV *sv;\n"
+        "%s" // arg_decls
         "    %s method;\n"
         "%s"
         "\n"
         "    CFISH_UNUSED_VAR(cv);\n"
         "    SP -= items;\n"
-        "    %s;\n"
+        "    if (%s) {\n"
+        "        XSBind_invalid_args_error(aTHX_ cv, \"%s\");\n"
+        "    }\n"
         "\n"
         "    /* Extract vars from Perl stack. */\n"
         "    %s\n"
-        "    %s\n"
+        "%s" // arg_assigns
         "\n"
         "    /* Execute */\n"
         "    %s\n"
         "}\n";
     char *xsub
         = CFCUtil_sprintf(pattern, self->sub.c_name, self->sub.c_name,
-                          self_type_c, arg_decls, meth_type_c, retval_decl,
-                          num_args_check, self_assign, var_assignments, body);
+                          arg_decls, meth_type_c, retval_decl, num_args_cond,
+                          xs_name_list, self_assign, arg_assigns, body);
 
-    FREEMEM(num_args_check);
-    FREEMEM(var_assignments);
+    FREEMEM(arg_assigns);
     FREEMEM(arg_decls);
     FREEMEM(meth_type_c);
     FREEMEM(self_assign);
     FREEMEM(body);
+    FREEMEM(num_args_cond);
+    FREEMEM(xs_name_list);
     FREEMEM(retval_decl);
     return xsub;
 }
 
 char*
-CFCPerlMethod_callback_def(CFCMethod *method) {
+CFCPerlMethod_callback_def(CFCMethod *method, CFCClass *klass) {
     CFCType *return_type = CFCMethod_get_return_type(method);
-    char *start = S_callback_start(method);
-    char *callback_def = NULL;
-    char *refcount_mods = S_callback_refcount_mods(method);
+    char *callback_body = NULL;
 
-    if (!start) {
-        // Can't map vars, because there's at least one type in the argument
-        // list we don't yet support.  Return a callback wrapper that throws
-        // an error error.
-        callback_def = S_invalid_callback_def(method);
-    }
-    else if (CFCType_is_void(return_type)) {
-        callback_def = S_void_callback_def(method, start, refcount_mods);
-    }
-    else if (CFCType_is_object(return_type)) {
-        callback_def = S_obj_callback_def(method, start, refcount_mods);
-    }
-    else if (CFCType_is_integer(return_type)
-             || CFCType_is_floating(return_type)
-        ) {
-        callback_def = S_primitive_callback_def(method, start, refcount_mods);
+    // Return a callback wrapper that throws an error if there are no
+    // bindings for a method.
+    if (!CFCMethod_can_be_bound(method)) {
+        callback_body = S_invalid_callback_body(method);
     }
     else {
-        // Can't map return type.
-        callback_def = S_invalid_callback_def(method);
+        char *start = S_callback_start(method);
+        char *refcount_mods = S_callback_refcount_mods(method);
+
+        if (CFCType_is_void(return_type)) {
+            callback_body = S_void_callback_body(method, start, refcount_mods);
+        }
+        else if (CFCType_is_object(return_type)) {
+            callback_body = S_obj_callback_body(method, start, refcount_mods);
+        }
+        else if (CFCType_is_integer(return_type)
+                 || CFCType_is_floating(return_type)
+            ) {
+            callback_body = S_primitive_callback_body(method, start,
+                                                      refcount_mods);
+        }
+        else {
+            // Can't map return type.
+            callback_body = S_invalid_callback_body(method);
+        }
+
+        FREEMEM(start);
+        FREEMEM(refcount_mods);
     }
 
-    FREEMEM(start);
-    FREEMEM(refcount_mods);
+    char *override_sym = CFCMethod_full_override_sym(method, klass);
+
+    CFCParamList *param_list = CFCMethod_get_param_list(method);
+    const char *params = CFCParamList_to_c(param_list);
+
+    const char *ret_type_str = CFCType_to_c(return_type);
+
+    char pattern[] =
+        "%s\n"
+        "%s(%s) {\n"
+        "%s"
+        "}\n";
+    char *callback_def
+        = CFCUtil_sprintf(pattern, ret_type_str, override_sym, params,
+                          callback_body);
+
+    FREEMEM(callback_body);
+    FREEMEM(override_sym);
     return callback_def;
 }
 
-static char*
-S_build_unused_vars(CFCVariable **vars) {
-    char *unused = CFCUtil_strdup("");
-
-    for (int i = 0; vars[i] != NULL; i++) {
-        const char *var_name = CFCVariable_micro_sym(vars[i]);
-        size_t size = strlen(unused) + strlen(var_name) + 80;
-        unused = (char*)REALLOCATE(unused, size);
-        strcat(unused, "\n    CFISH_UNUSED_VAR(");
-        strcat(unused, var_name);
-        strcat(unused, ");");
-    }
-
-    return unused;
-}
-
-static char*
-S_maybe_unreachable(CFCType *return_type) {
-    char *return_statement;
-    if (CFCType_is_void(return_type)) {
-        return_statement = CFCUtil_strdup("");
-    }
-    else {
-        const char *ret_type_str = CFCType_to_c(return_type);
-        char pattern[] = "\n    CFISH_UNREACHABLE_RETURN(%s);";
-        return_statement = CFCUtil_sprintf(pattern, ret_type_str);
-    }
-    return return_statement;
-}
-
 static char*
 S_callback_start(CFCMethod *method) {
     CFCParamList *param_list = CFCMethod_get_param_list(method);
     static const char pattern[] =
+        "    dTHX;\n"
         "    dSP;\n"
         "    EXTEND(SP, %d);\n"
         "    ENTER;\n"
@@ -512,7 +467,7 @@ S_callback_start(CFCMethod *method) {
     CFCVariable **arg_vars = CFCParamList_get_variables(param_list);
     for (int i = 1; arg_vars[i] != NULL; i++) {
         CFCVariable *var      = arg_vars[i];
-        const char  *name     = CFCVariable_micro_sym(var);
+        const char  *name     = CFCVariable_get_name(var);
         CFCType     *type     = CFCVariable_get_type(var);
         const char  *c_type   = CFCType_to_c(type);
 
@@ -524,15 +479,10 @@ S_callback_start(CFCMethod *method) {
                                  num_buf, ");\n", NULL);
         }
 
-        if (CFCType_is_string_type(type)) {
-            // Convert Clownfish string type to UTF-8 Perl string scalars.
-            params = CFCUtil_cat(params, "    mPUSHs(XSBind_str_to_sv(",
-                                 "(cfish_String*)", name, "));\n", NULL);
-        }
-        else if (CFCType_is_object(type)) {
-            // Wrap other Clownfish object types in Perl objects.
+        if (CFCType_is_object(type)) {
+            // Wrap Clownfish object types in Perl objects.
             params = CFCUtil_cat(params, "    mPUSHs(XSBind_cfish_to_perl(",
-                                 "(cfish_Obj*)", name, "));\n", NULL);
+                                 "aTHX_ (cfish_Obj*)", name, "));\n", NULL);
         }
         else if (CFCType_is_integer(type)) {
             // Convert primitive integer types to IV Perl scalars.
@@ -559,9 +509,9 @@ S_callback_start(CFCMethod *method) {
                                  name, ");\n", NULL);
         }
         else {
-            // Can't map variable type.  Signal to caller.
-            FREEMEM(params);
-            return NULL;
+            // Can't map variable type.
+            const char *type_str = CFCType_to_c(type);
+            CFCUtil_die("Can't map type '%s' to Perl", type_str);
         }
     }
 
@@ -583,7 +533,7 @@ S_callback_refcount_mods(CFCMethod *method) {
     // refcount.  (No function can return a decremented object.)
     if (CFCType_is_object(return_type) && !CFCType_incremented(return_type)) {
         refcount_mods = CFCUtil_cat(refcount_mods,
-                                    "\n    CFISH_DECREF(retval);", NULL);
+                                    "    CFISH_DECREF(retval);\n", NULL);
     }
 
     // Adjust refcounts of arguments per method signature, so that Perl code
@@ -591,17 +541,17 @@ S_callback_refcount_mods(CFCMethod *method) {
     for (int i = 0; arg_vars[i] != NULL; i++) {
         CFCVariable *var  = arg_vars[i];
         CFCType     *type = CFCVariable_get_type(var);
-        const char  *name = CFCVariable_micro_sym(var);
+        const char  *name = CFCVariable_get_name(var);
         if (!CFCType_is_object(type)) {
             continue;
         }
         else if (CFCType_incremented(type)) {
-            refcount_mods = CFCUtil_cat(refcount_mods, "\n    CFISH_INCREF(",
-                                        name, ");", NULL);
+            refcount_mods = CFCUtil_cat(refcount_mods, "    CFISH_INCREF(",
+                                        name, ");\n", NULL);
         }
         else if (CFCType_decremented(type)) {
-            refcount_mods = CFCUtil_cat(refcount_mods, "\n    CFISH_DECREF(",
-                                        name, ");", NULL);
+            refcount_mods = CFCUtil_cat(refcount_mods, "    CFISH_DECREF(",
+                                        name, ");\n", NULL);
         }
     }
 
@@ -609,60 +559,57 @@ S_callback_refcount_mods(CFCMethod *method) {
 }
 
 static char*
-S_invalid_callback_def(CFCMethod *method) {
-    char *full_method_sym = CFCMethod_full_method_sym(method, NULL);
-
-    const char *override_sym = CFCMethod_full_override_sym(method);
-    CFCParamList *param_list = CFCMethod_get_param_list(method);
-    const char *params = CFCParamList_to_c(param_list);
+S_invalid_callback_body(CFCMethod *method) {
+    CFCParamList *param_list   = CFCMethod_get_param_list(method);
     CFCVariable **param_vars = CFCParamList_get_variables(param_list);
+    char *unused = CFCUtil_strdup("");
+    for (int i = 0; param_vars[i] != NULL; i++) {
+        const char *name = CFCVariable_get_name(param_vars[i]);
+        unused = CFCUtil_cat(unused, "    CFISH_UNUSED_VAR(", name, ");\n",
+                             NULL);
+    }
 
-    // Thwart compiler warnings.
     CFCType *return_type = CFCMethod_get_return_type(method);
     const char *ret_type_str = CFCType_to_c(return_type);
-    char *unused = S_build_unused_vars(param_vars);
-    char *unreachable = S_maybe_unreachable(return_type);
+    char *maybe_ret
+        = CFCType_is_void(return_type)
+          ? CFCUtil_sprintf("")
+          : CFCUtil_sprintf("    CFISH_UNREACHABLE_RETURN(%s);\n",
+                            ret_type_str);
+
+    char *perl_name = CFCPerlMethod_perl_name(method);
 
     char pattern[] =
-        "%s\n"
-        "%s(%s) {%s\n"
-        "    CFISH_THROW(CFISH_ERR, \"Can't override %s via binding\");%s\n"
-        "}\n";
-    char *callback_def
-        = CFCUtil_sprintf(pattern, ret_type_str, override_sym, params, unused,
-                          full_method_sym, unreachable);
-
-    FREEMEM(full_method_sym);
-    FREEMEM(unreachable);
-    FREEMEM(unused);
-    return callback_def;
-}
-
-static char*
-S_void_callback_def(CFCMethod *method, const char *callback_start,
-                    const char *refcount_mods) {
-    const char *override_sym = CFCMethod_full_override_sym(method);
-    const char *params = CFCParamList_to_c(CFCMethod_get_param_list(method));
-    char *perl_name = CFCPerlMethod_perl_name(method);
-    const char pattern[] =
-        "void\n"
-        "%s(%s) {\n"
         "%s"
-        "    S_finish_callback_void(\"%s\");%s\n"
-        "}\n";
-    char *callback_def
-        = CFCUtil_sprintf(pattern, override_sym, params, callback_start,
-                          perl_name, refcount_mods);
+        "    cfish_Err_invalid_callback(\"%s\");\n"
+        "%s";
+    char *callback_body
+        = CFCUtil_sprintf(pattern, unused, perl_name, maybe_ret);
 
     FREEMEM(perl_name);
-    return callback_def;
+    FREEMEM(unused);
+    FREEMEM(maybe_ret);
+    return callback_body;
 }
 
 static char*
-S_primitive_callback_def(CFCMethod *method, const char *callback_start,
-                         const char *refcount_mods) {
-    const char *override_sym = CFCMethod_full_override_sym(method);
-    const char *params = CFCParamList_to_c(CFCMethod_get_param_list(method));
+S_void_callback_body(CFCMethod *method, const char *callback_start,
+                     const char *refcount_mods) {
+    char *perl_name = CFCPerlMethod_perl_name(method);
+    const char pattern[] =
+        "%s"
+        "    S_finish_callback_void(aTHX_ \"%s\");\n"
+        "%s";
+    char *callback_body
+        = CFCUtil_sprintf(pattern, callback_start, perl_name, refcount_mods);
+
+    FREEMEM(perl_name);
+    return callback_body;
+}
+
+static char*
+S_primitive_callback_body(CFCMethod *method, const char *callback_start,
+                          const char *refcount_mods) {
     CFCType *return_type = CFCMethod_get_return_type(method);
     const char *ret_type_str = CFCType_to_c(return_type);
     char callback_func[50];
@@ -685,26 +632,21 @@ S_primitive_callback_def(CFCMethod *method, const char *callback_start,
     char *perl_name = CFCPerlMethod_perl_name(method);
 
     char pattern[] =
-        "%s\n"
-        "%s(%s) {\n"
         "%s"
-        "    %s retval = (%s)%s(\"%s\");%s\n"
-        "    return retval;\n"
-        "}\n";
-    char *callback_def
-        = CFCUtil_sprintf(pattern, ret_type_str, override_sym, params,
-                          callback_start, ret_type_str, ret_type_str,
+        "    %s retval = (%s)%s(aTHX_ \"%s\");\n"
+        "%s"
+        "    return retval;\n";
+    char *callback_body
+        = CFCUtil_sprintf(pattern, callback_start, ret_type_str, ret_type_str,
                           callback_func, perl_name, refcount_mods);
 
     FREEMEM(perl_name);
-    return callback_def;
+    return callback_body;
 }
 
 static char*
-S_obj_callback_def(CFCMethod *method, const char *callback_start,
-                   const char *refcount_mods) {
-    const char *override_sym = CFCMethod_full_override_sym(method);
-    const char *params = CFCParamList_to_c(CFCMethod_get_param_list(method));
+S_obj_callback_body(CFCMethod *method, const char *callback_start,
+                    const char *refcount_mods) {
     CFCType *return_type = CFCMethod_get_return_type(method);
     const char *ret_type_str = CFCType_to_c(return_type);
     const char *nullable  = CFCType_nullable(return_type) ? "true" : "false";
@@ -712,18 +654,15 @@ S_obj_callback_def(CFCMethod *method, const char *callback_start,
     char *perl_name = CFCPerlMethod_perl_name(method);
 
     char pattern[] =
-        "%s\n"
-        "%s(%s) {\n"
         "%s"
-        "    %s retval = (%s)S_finish_callback_obj(self, \"%s\", %s);%s\n"
-        "    return retval;\n"
-        "}\n";
-    char *callback_def
-        = CFCUtil_sprintf(pattern, ret_type_str, override_sym, params,
-                          callback_start, ret_type_str, ret_type_str,
+        "    %s retval = (%s)S_finish_callback_obj(aTHX_ self, \"%s\", %s);\n"
+        "%s"
+        "    return retval;\n";
+    char *callback_body
+        = CFCUtil_sprintf(pattern, callback_start, ret_type_str, ret_type_str,
                           perl_name, nullable, refcount_mods);
 
     FREEMEM(perl_name);
-    return callback_def;
+    return callback_body;
 }
 
